==> helpdesk.ron.cz?menuid=download&filename=ronportal_4_2107_155.zip: ronportal/ron/mvc/downloadfileoutputobject.php <==
<?php
/**
 * stanislav.elbl, 19.02.2021
 */

namespace Ron\MVC;

use Exception;
use Ron\Http\HttpResponse;
use Ron\TextManager;

/**
 * Provede stahnuti souboru z filesystemu.
 * Class DownloadFileOutputObject
 * @package Ron\MVC
 */
class DownloadFileOutputObject implements OutputObject
{
    public $path;
    public $fileName;
    public $outputFileName;
    public $fileSize;

    public $fullName;

    public $charset;

    /**
     * DownloadFileOutputObject constructor.
     * @param string $path
     * @param string $fileName Jmeno souboru nesmi obsahovat nektere specialni znaky (predevsim / a \). Pokud je null, musi byt zadano uz v $path
     * @param string $outputFileName Jmeno souboru na vystupu. Pokud neni zadano, pouzije se $fileName
     * @param string $charset
     * @throws Exception
     */
    public function __construct($path, $fileName, $outputFileName = null, $charset = null)
    {
        if (strpos($fileName, '\\') !== false   ||   strpos($fileName, '/')) {
            throw new Exception('DownloadFileOutputObject: jmeno souboru nesmi obsahovat lomitko!');
        }

        $this->path = $path;
        $this->fileName = $fileName;
        $this->outputFileName = $outputFileName ?: $fileName;

        $this ->fullName = $path ?: '';
        $this ->charset = $charset;

        if (isset($fileName)) {
            $lastChar = substr($path, -1);
            if ($path && $lastChar !== '\\' && $lastChar !== '/') {
                // doplni na konec cesty lomitko, pokud tam neni
                $this->fullName .= '/';
            }

            $this->fullName .= $this->fileName;
        }

        if (!file_exists($this ->fullName)) {
            throw new Exception( TextManager::_('File_not_exists!') );
        }

        // Velikost souboru nacte jen kdyz je zadano jmeno souboru
        $this ->fileSize = $this ->fileName ? filesize($this ->fullName) : null;
    }


    public function output()
    {
        ob_end_clean(); // vypne output buffering, aby nebyla velikost souboru omezena memory_limitem
        
        HttpResponse::getInstance() ->getHeaders() ->addHeadersForContent( $this ->outputFileName, $this ->fileSize, $this ->charset );
        HttpResponse::renderHeaders();

        readfile($this ->fullName);
    }
}

==> helpdesk.ron.cz?menuid=download&filename=ronportal_4_2311_175.zip: ronportal/ron/mvc/downloadfileoutputobject.php <==
<?php
/**
 * stanislav.elbl, 19.02.2021
 */

namespace Ron\MVC;

use Exception;
use Ron\Http\HttpResponse;
use Ron\TextManager;

/**
 * Generuje na vystup soubor z filesystemu.
 */
class DownloadFileOutputObject implements OutputObject
{
    /** @var string     Cesta k souboru ve filesystemu */
    public $path;

    /** @var string     Jmeno souboru. */
    public $fileName;

    /** @var string     Jmeno vystupniho souboru (vygeneruje se do HTTP hlavicky) - muze se lisit od puvodniho jmena souboru. Pokud neni zadano, tak se pouzije puvodni jmeno souboru. */
    public $outputFileName;

    /** @var false|int|null     Velikost souboru (pokud neni nastavena na false vygeneruje se do HTTP hlavicky). */
    public $fileSize;

    /** @var string     Nastavi se podle $path a $fileName */
    public $fullName;

    public $charset;

    /**
     * DownloadFileOutputObject constructor.
     * @param string $path      Cesta k souboru
     * @param string $fileName  Jmeno souboru nesmi obsahovat nektere specialni znaky (predevsim / a \). Pokud je null, musi byt zadano uz v $path
     * @param string $outputFileName    Jmeno souboru na vystupu. Pokud neni zadano, pouzije se $fileName
     * @param string $charset   Znakova sada (generuje se do HTTP hlavicky)
     * @throws Exception
     */
    public function __construct($path, $fileName, $outputFileName = null, $charset = null)
    {
        if (strpos($fileName, '\\') !== false   ||   strpos($fileName, '/')) {
            throw new Exception('DownloadFileOutputObject: jmeno souboru nesmi obsahovat lomitko!');
        }

        $this->path = $path;
        $this->fileName = $fileName;
        $this->outputFileName = $outputFileName ?: $fileName;

        $this ->fullName = $path ?: '';
        $this ->charset = $charset;

        if (isset($fileName)) {
            $lastChar = substr($path, -1);
            if ($path && $lastChar !== '\\' && $lastChar !== '/') {
                // doplni na konec cesty lomitko, pokud tam neni
                $this->fullName .= '/';
            }

            $this->fullName .= $this->fileName;
        }

        if (!file_exists($this ->fullName)) {
            throw new Exception( TextManager::_('File_not_exists!') );
        }

        // Velikost souboru nacte jen kdyz je zadano jmeno souboru
        $this ->fileSize = $this ->fileName ? filesize($this ->fullName) : null;
    }


    public function output()
    {
        ob_end_clean(); // vypne output buffering, aby nebyla velikost souboru omezena memory_limitem
        
        HttpResponse::getInstance() ->getHeaders() ->addHeadersForContent( $this ->outputFileName, $this ->fileSize, $this ->charset );
        HttpResponse::renderHeaders();

        readfile($this ->fullName);
    }
}
